@@ -426,6 +426,11 @@ term_expansion(safe_primitive(Goal), Term) :-
     ->  Term = safe_primitive(Goal)
     ;   Term = []
     ).
+term_expansion((safe_primitive(Goal) :- _), Term) :-
+    (   verify_safe_declaration(Goal)
+    ->  Term = safe_primitive(Goal)
+    ;   Term = []
+    ).
 
 system:term_expansion(sandbox:safe_primitive(Goal), Term) :-
     \+ current_prolog_flag(xref, true),
@@ -433,6 +438,12 @@ system:term_expansion(sandbox:safe_primitive(Goal), Term) :-
     ->  Term = sandbox:safe_primitive(Goal)
     ;   Term = []
     ).
+system:term_expansion((sandbox:safe_primitive(Goal) :- _), Term) :-
+    \+ current_prolog_flag(xref, true),
+    (   verify_safe_declaration(Goal)
+    ->  Term = sandbox:safe_primitive(Goal)
+    ;   Term = []
+    ).
 
 verify_safe_declaration(Var) :-
     var(Var),
@@ -462,13 +473,15 @@ verify_safe_declaration(Goal) :-
     ).
 
 ok_meta(system:assert(_)).
+ok_meta(system:load_files(_,_)).
 ok_meta(system:use_module(_,_)).
 ok_meta(system:use_module(_)).
 
 verify_predefined_safe_declarations :-
     forall(clause(safe_primitive(Goal), _Body, Ref),
-           ( catch(verify_safe_declaration(Goal), E, true),
-             (   nonvar(E)
+           ( E = error(F,_),
+             catch(verify_safe_declaration(Goal), E, true),
+             (   nonvar(F)
              ->  clause_property(Ref, file(File)),
                  clause_property(Ref, line_count(Line)),
                  print_message(error, bad_safe_declaration(Goal, File, Line))
@@ -512,6 +525,7 @@ safe_primitive(atomic(_)).
 safe_primitive(compound(_)).
 safe_primitive(callable(_)).
 safe_primitive(ground(_)).
+safe_primitive(system:nonground(_,_)).
 safe_primitive(system:cyclic_term(_)).
 safe_primitive(acyclic_term(_)).
 safe_primitive(system:is_stream(_)).
@@ -563,6 +577,7 @@ safe_primitive(system:'$filled_array'(_,_,_,_)).
 safe_primitive(copy_term(_,_)).
 safe_primitive(system:duplicate_term(_,_)).
 safe_primitive(system:copy_term_nat(_,_)).
+safe_primitive(system:size_abstract_term(_,_,_)).
 safe_primitive(numbervars(_,_,_)).
 safe_primitive(system:numbervars(_,_,_,_)).
 safe_primitive(subsumes_term(_,_)).
@@ -742,7 +757,7 @@ safe_primitive('$tabling':abolish_all_tables).
 safe_primitive('$tabling':'$wrap_tabled'(Module:_Head, _Mode)) :-
     prolog_load_context(module, Module),
     !.
-safe_primitive('$tabling':'$moded_wrap_tabled'(Module:_Head,_,_,_)) :-
+safe_primitive('$tabling':'$moded_wrap_tabled'(Module:_Head,_,_,_,_)) :-
     prolog_load_context(module, Module),
     !.
 
@@ -752,6 +767,9 @@ safe_primitive('$tabling':'$moded_wrap_tabled'(Module:_Head,_,_,_)) :-
 
 safe_primitive(system:use_module(Spec, _Import)) :-
     safe_primitive(system:use_module(Spec)).
+safe_primitive(system:load_files(Spec, Options)) :-
+    safe_primitive(system:use_module(Spec)),
+    maplist(safe_load_file_option, Options).
 safe_primitive(system:use_module(Spec)) :-
     ground(Spec),
     (   atom(Spec)
@@ -783,6 +801,12 @@ segments_to_path(X) -->
 
 save_extension(pl).
 
+safe_load_file_option(if(changed)).
+safe_load_file_option(if(not_loaded)).
+safe_load_file_option(must_be_module(_)).
+safe_load_file_option(optimise(_)).
+safe_load_file_option(silent(_)).
+
 %!  safe_assert(+Term) is semidet.
 %
 %   True if assert(Term) is safe,  which   means  it  asserts in the
@@ -1145,6 +1169,7 @@ safe_pattr(multifile(_)).
 safe_pattr(public(_)).
 safe_pattr(meta_predicate(_)).
 safe_pattr(table(_)).
+safe_pattr(non_terminal(_)).
 
 safe_pattr(Var, _) :-
     var(Var),
@@ -1181,6 +1206,7 @@ safe_source_directive(initialization(_,_)). % Checked at runtime
 
 directive_loads_file(use_module(library(X)), X).
 directive_loads_file(use_module(library(X), _Imports), X).
+directive_loads_file(load_files(library(X), _Options), X).
 directive_loads_file(ensure_loaded(library(X)), X).
 directive_loads_file(include(X), X).
 
@@ -1235,6 +1261,10 @@ safe_prolog_flag(max_rational_size_action, _).
 % tabling
 safe_prolog_flag(max_answers_for_subgoal,_).
 safe_prolog_flag(max_answers_for_subgoal_action,_).
+safe_prolog_flag(max_table_answer_size,_).
+safe_prolog_flag(max_table_answer_size_action,_).
+safe_prolog_flag(max_table_subgoal_size,_).
+safe_prolog_flag(max_table_subgoal_size_action,_).
 
 
 %!  prolog:sandbox_allowed_expansion(:G) is det.
