@@ -156,6 +156,8 @@ struct PL_global_data
     int		engines_finished;	/* # engines threads */
     double	thread_cputime;		/* Total CPU time of threads */
 #endif
+    int		errors;			/* Printed error messages */
+    int		warnings;		/* Printed warning messages */
   } statistics;
 
 #ifdef O_PROFILE
@@ -440,7 +442,6 @@ struct PL_local_data
 #endif
   int		in_arithmetic;		/* doing arithmetic */
   int		in_print_message;	/* Inside printMessage() */
-  gen_t		gen_reload;		/* reload generation */
   void *	glob_info;		/* pl-glob.c */
   IOENC		encoding;		/* default I/O encoding */
   struct PL_local_data *next_free;	/* see maybe_free_local_data() */
@@ -521,6 +522,8 @@ struct PL_local_data
     double	last_walltime;		/* Last Wall time (m-secs since start) */
     double	user_cputime;		/* User saved CPU time */
     double	system_cputime;		/* Kernel saved CPU time */
+    int		errors;			/* Printed error messages */
+    int		warnings;		/* Printed warning messages */
   } statistics;
 
 #ifdef O_GMP
@@ -738,11 +741,22 @@ struct PL_local_data
     int64_t	cgc_inferences;		/* Inferences at last cgc consider */
   } clauses;
 
+  struct
+  { gen_t	generation;		/* reload generation */
+    int		nesting;		/* reload nesting */
+  } reload;
+
   struct
   { DefinitionChain nesting;		/* Nesting chain in the autoloader */
     Definition	loop;			/* We are looping on this def */
   } autoload;
 
+  struct
+  { term_t undo_list;			/* Stacked undo goals */
+    Buffer scheduled;
+    int    running;
+  } undo;
+
   struct
   { intptr_t _total_marked;		/* # marked global cells */
     intptr_t _trailcells_deleted;	/* # garbage trailcells */
