@@ -2137,9 +2137,16 @@ clear_frame_vars(LocalFrame target)
 
     if ( ch )
     { ch = ch->parent;
-      if ( ch->parent )
+      if ( ch )
       { fr = ch->frame;
-	PC = ch->type == CHP_JUMP ? ch->value.PC : NULL;
+	if ( ch->type == CHP_JUMP )
+	{ Code codes = fr->clause->value.clause->codes;	     \
+
+	  PC = ch->value.PC;
+	  assert(PC < &codes[codes[-1]]);
+	} else
+	  PC = NULL;
+
 	continue;
       }
     }
