@@ -45,6 +45,7 @@
 #include "pl-prof.h"
 #include "pl-event.h"
 #include "pl-tabling.h"
+#include "pl-undo.h"
 #include <fenv.h>
 #ifdef _MSC_VER
 #pragma warning(disable: 4102)		/* unreferenced labels */
@@ -283,6 +284,7 @@ updateAlerted(PL_local_data_t *ld)
   if ( ld->_debugstatus.debugging )		mask |= ALERT_DEBUG;
 #endif
   if ( ld->fli.string_buffers.top )		mask |= ALERT_BUFFER;
+  if ( UNDO_SCHEDULED(ld) )			mask |= ALERT_UNDO;
 
   ld->alerted = mask;
 
@@ -1458,6 +1460,8 @@ __do_undo(mark *m ARG_LD)
     if ( isTrailVal(p) )
     { DEBUG(2, Sdprintf("Undoing a trailed assignment\n"));
       tt--;
+      if ( tt->address == gBase )
+	push_undo(tt->address PASS_LD);
       *tt->address = trailVal(p);
       DEBUG(CHK_SECURE,
 	    if ( isAttVar(*tt->address) )
@@ -2509,7 +2513,7 @@ one has a programPointer pointing to  I_EXITQUERY, doing the return from
 a query.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define NDEBUG_SAVE_FLAGS (PLFLAG_LASTCALL)
+#define NDEBUG_SAVE_FLAGS prologFlagMask(PLFLAG_LASTCALL)
 
 qid_t
 PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
@@ -2638,7 +2642,7 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
     suspendTrace(TRUE);
     qf->debugSave = debugstatus.debugging;
     debugstatus.debugging = DBG_OFF;
-    qf->flags_saved = (LD->prolog_flag.mask.flags & NDEBUG_SAVE_FLAGS);
+    qf->flags_saved = (LD->prolog_flag.mask.flags[0] & NDEBUG_SAVE_FLAGS);
     setPrologFlagMask(PLFLAG_LASTCALL);
 #ifdef O_LIMIT_DEPTH
     qf->saved_depth_limit   = LD->depth_info.limit;
@@ -2717,8 +2721,8 @@ restore_after_query(QueryFrame qf)
   if ( true(qf, PL_Q_NODEBUG) )
   { suspendTrace(FALSE);
     debugstatus.debugging = qf->debugSave;
-    LD->prolog_flag.mask.flags &= (~NDEBUG_SAVE_FLAGS);
-    LD->prolog_flag.mask.flags |= qf->flags_saved;
+    LD->prolog_flag.mask.flags[0] &= (~NDEBUG_SAVE_FLAGS);
+    LD->prolog_flag.mask.flags[0] |= qf->flags_saved;
 #ifdef O_LIMIT_DEPTH
     LD->depth_info.limit   = qf->saved_depth_limit;
     LD->depth_info.reached = qf->saved_depth_reached;
@@ -3319,9 +3323,20 @@ next_choice:
   last_choice = ch->type;
 #endif
 
-  if ( (LD->alerted & ALERT_BUFFER) )
-  { LD->alerted &= ~ALERT_BUFFER;
-    release_string_buffers_from_frame(FR PASS_LD);
+  if ( LD->alerted )
+  { if ( LD->alerted & ALERT_BUFFER )
+    { LD->alerted &= ~ALERT_BUFFER;
+      release_string_buffers_from_frame(FR PASS_LD);
+    }
+    if ( UNDO_SCHEDULED(LD) )
+    { int rc;
+
+      SAVE_REGISTERS(qid);
+      rc = run_undo_hooks(PASS_LD1);
+      LOAD_REGISTERS(qid);
+      if ( !rc )
+	THROW_EXCEPTION;
+    }
   }
 
   switch(ch->type)
